@@ -6,8 +6,10 @@
 import os.path
 import sys
 import signal
+import errno
 from osc import oscerr
 from urllib2 import URLError, HTTPError
+from httplib import HTTPException, BadStatusLine
 from oscsslexcp import NoSecureSSLError
 from osc.util.cpio import CpioError
 from osc.util.packagequery import PackageError
@@ -122,10 +124,31 @@ def run(prg):
 
         return 1
 
+    except BadStatusLine, e:
+        print >>sys.stderr, 'Server returned an invalid response:', e
+        print >>sys.stderr, e.line
+        return 1
+
+    except HTTPException, e:
+        print >>sys.stderr, e
+        return 1
+
     except URLError, e:
         print >>sys.stderr, 'Failed to reach a server:\n', e.reason
         return 1
 
+    except IOError, e:
+        # ignore broken pipe
+        if e.errno != errno.EPIPE:
+            raise
+        return 1
+
+    except OSError, e:
+        if e.errno != errno.ENOENT:
+            raise
+        print >>sys.stderr, e
+        return 1
+
     except (oscerr.ConfigError, oscerr.NoConfigfile), e:
         print >>sys.stderr, e.msg
         return 1
@@ -160,6 +183,7 @@ def run(prg):
             'and the following traceback to it:'
         print >>sys.stderr, e.msg
         traceback.print_exc(file=sys.stderr)
+        return 1
 
     except oscerr.PackageError, e:
         print >>sys.stderr, e.msg
@@ -189,4 +213,8 @@ def run(prg):
         print >>sys.stderr, e
         return 1
 
+    except oscerr.OscBaseError, e:
+        print >>sys.stderr, '*** Error:', e
+        return 1
+
 # vim: sw=4 et
