@@ -9,6 +9,7 @@ import os
 import re
 import sys
 import shutil
+import urlparse
 from tempfile import NamedTemporaryFile, mkdtemp
 from osc.fetch import *
 from osc.core import get_buildinfo, store_read_apiurl, store_read_project, store_read_package, meta_exists, quote_plus, get_buildconfig, is_package_dir
@@ -41,15 +42,16 @@ can_also_build = {
              'armv6l' :[                                         'armv4l', 'armv5el', 'armv6el'                       ],
              'armv7el':[                                         'armv4l', 'armv5el', 'armv6el', 'armv7el'            ],
              'armv7l' :[                                         'armv4l', 'armv5el', 'armv6el', 'armv7el'            ],
+             'armv7hl':[                        'armv7hl'                                                             ],
              'armv8el':[                                         'armv4l', 'armv5el', 'armv6el', 'armv7el', 'armv8el' ],
              'armv8l' :[                                         'armv4l', 'armv5el', 'armv6el', 'armv7el', 'armv8el' ],
              's390x':  ['s390' ],
              'ppc64':  [                        'ppc', 'ppc64' ],
              'sh4':    [                                                                                               'sh4' ],
-             'i386':   [        'i586',         'ppc', 'ppc64',  'armv4l', 'armv5el', 'armv6el', 'armv7el', 'armv8el', 'sh4', 'mips', 'mipsel' ],
-             'i586':   [                'i386', 'ppc', 'ppc64',  'armv4l', 'armv5el', 'armv6el', 'armv7el', 'armv8el', 'sh4', 'mips', 'mipsel' ],
-             'i686':   [        'i586',         'ppc', 'ppc64',  'armv4l', 'armv5el', 'armv6el', 'armv7el', 'armv8el', 'sh4', 'mips', 'mipsel' ],
-             'x86_64': ['i686', 'i586', 'i386', 'ppc', 'ppc64',  'armv4l', 'armv5el', 'armv6el', 'armv7el', 'armv8el', 'sh4', 'mips', 'mipsel' ],
+             'i386':   [        'i586',         'ppc', 'ppc64',  'armv4l', 'armv5el', 'armv6el', 'armv7el', 'armv7hl', 'armv8el', 'sh4', 'mips', 'mipsel' ],
+             'i586':   [                'i386', 'ppc', 'ppc64',  'armv4l', 'armv5el', 'armv6el', 'armv7el', 'armv7hl', 'armv8el', 'sh4', 'mips', 'mipsel' ],
+             'i686':   [        'i586',         'ppc', 'ppc64',  'armv4l', 'armv5el', 'armv6el', 'armv7el', 'armv7hl', 'armv8el', 'sh4', 'mips', 'mipsel' ],
+             'x86_64': ['i686', 'i586', 'i386', 'ppc', 'ppc64',  'armv4l', 'armv5el', 'armv6el', 'armv7el', 'armv7hl', 'armv8el', 'sh4', 'mips', 'mipsel' ],
              'sparc64': ['sparc64v', 'sparcv9v', 'sparcv9', 'sparcv8', 'sparc'],
              'parisc': ['hppa'],
              }
@@ -122,8 +124,6 @@ class Buildinfo:
         self.vminstall_list = [ dep.name for dep in self.deps if dep.vminstall ]
         self.cbinstall_list = [ dep.name for dep in self.deps if dep.cbinstall ]
         self.cbpreinstall_list = [ dep.name for dep in self.deps if dep.cbpreinstall ]
-        self.cbpreignore_list = [ dep.name for dep in self.deps if dep.cbpreignore ]
-        self.cbignore_list = [ deb.name for dep in self.deps if dep.cbignore ]
         self.preinstall_list = [ dep.name for dep in self.deps if dep.preinstall ]
         self.runscripts_list = [ dep.name for dep in self.deps if dep.runscripts ]
 
@@ -155,7 +155,7 @@ class Pac:
                   'version', 'release',
                   'project', 'repository',
                   'preinstall', 'vminstall', 'noinstall', 'runscripts',
-                  'cbinstall', 'cbpreinstall', 'cbpreignore', 'cbignore',
+                  'cbinstall', 'cbpreinstall',
                  ]:
             self.mp[i] = node.get(i)
 
@@ -359,6 +359,7 @@ def main(apiurl, opts, argv):
     build_descr = argv[2]
     xp = []
     build_root = None
+    cache_dir  = None
     build_uid=''
     vm_type = config['build-type']
 
@@ -385,9 +386,9 @@ def main(apiurl, opts, argv):
     if opts.root:
         build_root = opts.root
     if opts.jobs:
-        buildargs.append('--jobs %s' % opts.jobs)
+        buildargs.append('--jobs=%s' % opts.jobs)
     elif config['build-jobs'] > 1:
-        buildargs.append('--jobs %s' % config['build-jobs'])
+        buildargs.append('--jobs=%s' % config['build-jobs'])
     if opts.icecream or config['icecream'] != '0':
         if opts.icecream:
             num = opts.icecream
@@ -395,7 +396,7 @@ def main(apiurl, opts, argv):
             num = config['icecream']
 
         if int(num) > 0:
-            buildargs.append('--icecream %s' % num)
+            buildargs.append('--icecream=%s' % num)
             xp.append('icecream')
             xp.append('gcc-c++')
     if opts.ccache:
@@ -409,13 +410,13 @@ def main(apiurl, opts, argv):
         buildargs.append('--debug')
     if opts._with:
         for o in opts._with:
-            buildargs.append('--with %s' % o)
+            buildargs.append('--with=%s' % o)
     if opts.without:
         for o in opts.without:
-            buildargs.append('--without %s' % o)
-# FIXME: quoting
-#    if opts.define:
-#        buildargs.append('--define "%s"' % opts.define)
+            buildargs.append('--without=%s' % o)
+    if opts.define:
+        for o in opts.define:
+            buildargs.append('--define=%s' % o)
     if config['build-uid']:
         build_uid = config['build-uid']
     if opts.build_uid:
@@ -423,9 +424,9 @@ def main(apiurl, opts, argv):
     if build_uid:
         buildidre = re.compile('^[0-9]{1,5}:[0-9]{1,5}$')
         if build_uid == 'caller':
-            buildargs.append('--uid %s:%s' % (os.getuid(), os.getgid()))
+            buildargs.append('--uid=%s:%s' % (os.getuid(), os.getgid()))
         elif buildidre.match(build_uid):
-            buildargs.append('--uid %s' % build_uid)
+            buildargs.append('--uid=%s' % build_uid)
         else:
             print >>sys.stderr, 'Error: build-uid arg must be 2 colon separated numerics: "uid:gid" or "caller"'
             return 1
@@ -440,6 +441,8 @@ def main(apiurl, opts, argv):
             pac = '_repository'
         else:
             pac = store_read_package(os.curdir)
+    if opts.shell:
+        buildargs.append("--shell")
 
     # make it possible to override configuration of the rc file
     for var in ['OSC_PACKAGECACHEDIR', 'OSC_SU_WRAPPER', 'OSC_BUILD_ROOT']:
@@ -460,10 +463,12 @@ def main(apiurl, opts, argv):
                 opts.local_package = True
         if opts.local_package:
             pacname = os.path.splitext(build_descr)[0]
+    apihost = urlparse.urlsplit(apiurl)[1]
     if not build_root:
-        build_root = config['build-root'] % { 'repo': repo, 'arch': arch,
-                                                    'project' : prj, 'package' : pacname
-                                                  }
+        build_root = config['build-root'] % {'repo': repo, 'arch': arch,
+            'project': prj, 'package': pacname, 'apihost': apihost}
+
+    cache_dir = config['packagecachedir'] % {'apihost': apihost}
 
     extra_pkgs = []
     if not opts.extra_pkgs:
@@ -490,6 +495,11 @@ def main(apiurl, opts, argv):
             s += "%%define _without_%s 0\n" % i
             s += "%%define _with_%s 1\n" % i
         build_descr_data = s + build_descr_data
+    if opts.define:
+        s = ''
+        for i in opts.define:
+            s += "%%define %s\n" % i
+        build_descr_data = s + build_descr_data
 
     if opts.prefer_pkgs:
         print 'Scanning the following dirs for local packages: %s' % ', '.join(opts.prefer_pkgs)
@@ -498,7 +508,7 @@ def main(apiurl, opts, argv):
         build_descr_data = cpio.get()
 
     # special handling for overlay and rsync-src/dest
-    specialcmdopts = ''
+    specialcmdopts = []
     if opts.rsyncsrc or opts.rsyncdest :
         if not opts.rsyncsrc or not opts.rsyncdest:
             raise oscerr.WrongOptions('When using --rsync-{src,dest} both parameters have to be specified.')
@@ -509,12 +519,12 @@ def main(apiurl, opts, argv):
         myrsyncdest = os.path.expandvars(opts.rsyncdest)
         if not os.path.isabs(myrsyncdest):
             raise oscerr.WrongOptions('--rsync-dest %s is no absolute path (starting with \'/\')!' % opts.rsyncdest)
-        specialcmdopts = '--rsync-src="%s" --rsync-dest="%s"' % (myrsyncsrc, myrsyncdest)
+        specialcmdopts = ['--rsync-src='+myrsyncsrc, '--rsync-dest='+myrsyncdest]
     if opts.overlay:
         myoverlay = os.path.abspath(os.path.expanduser(os.path.expandvars(opts.overlay)))
         if not os.path.isdir(myoverlay):
             raise oscerr.WrongOptions('--overlay %s is no valid directory!' % opts.overlay)
-        specialcmdopts += '--overlay="%s"' % myoverlay
+        specialcmdopts += ['--overlay='+myoverlay]
 
     bi_file = None
     bc_file = None
@@ -548,8 +558,6 @@ def main(apiurl, opts, argv):
                 raise oscerr.WrongOptions('--offline is not possible, no local buildconfig file')
         else:
             print 'Getting buildinfo from server and store to %s' % bi_filename
-            if not bi_file:
-                bi_file = open(bi_filename, 'w')
             bi_text = ''.join(get_buildinfo(apiurl,
                                             prj,
                                             pac,
@@ -557,12 +565,16 @@ def main(apiurl, opts, argv):
                                             arch,
                                             specfile=build_descr_data,
                                             addlist=extra_pkgs))
+            if not bi_file:
+                bi_file = open(bi_filename, 'w')
+            # maybe we should check for errors before saving the file
             bi_file.write(bi_text)
             bi_file.flush()
             print 'Getting buildconfig from server and store to %s' % bc_filename
+            bc = get_buildconfig(apiurl, prj, repo)
             if not bc_file:
                 bc_file = open(bc_filename, 'w')
-            bc_file.write(get_buildconfig(apiurl, prj, repo))
+            bc_file.write(bc)
             bc_file.flush()
     except urllib2.HTTPError, e:
         if e.code == 404:
@@ -599,7 +611,7 @@ def main(apiurl, opts, argv):
         bi.release = opts.release
 
     if bi.release:
-        buildargs.append('--release %s' % bi.release)
+        buildargs.append('--release=%s' % bi.release)
 
     # real arch of this machine
     # vs.
@@ -641,7 +653,7 @@ def main(apiurl, opts, argv):
     if opts.disable_cpio_bulk_download:
         urllist.append( '%(apiurl)s/build/%(project)s/%(repository)s/%(repoarch)s/%(repopackage)s/%(repofilename)s' )
 
-    fetcher = Fetcher(cachedir = config['packagecachedir'],
+    fetcher = Fetcher(cache_dir,
                       urllist = urllist,
                       api_host_options = config['api_host_options'],
                       offline = opts.noinit or opts.offline,
@@ -675,7 +687,7 @@ def main(apiurl, opts, argv):
                 for i in range(0, len(a)):
                     data[i] = a[i]
 
-            destdir = os.path.join(config['packagecachedir'], data[0], data[2], data[3])
+            destdir = os.path.join(cache_dir, data[0], data[2], data[3])
             old_pkg_dir = None
             try:
                 print "Downloading previous build from %s ..." % '/'.join(data)
@@ -718,7 +730,7 @@ def main(apiurl, opts, argv):
                                 progress_meter = True)
 
         if old_pkg_dir != None:
-            buildargs.append('--oldpackages %s' % old_pkg_dir)
+            buildargs.append('--oldpackages=%s' % old_pkg_dir)
 
     # Make packages from buildinfo available as repos for kiwi
     if build_type == 'kiwi':
@@ -777,8 +789,6 @@ def main(apiurl, opts, argv):
     rpmlist.append('vminstall: ' + ' '.join(bi.vminstall_list) + '\n')
     rpmlist.append('cbinstall: ' + ' '.join(bi.cbinstall_list) + '\n')
     rpmlist.append('cbpreinstall: ' + ' '.join(bi.cbpreinstall_list) + '\n')
-    rpmlist.append('cbpreignore: ' + ' '.join(bi.cbpreinstall_list) + '\n')
-    rpmlist.append('cbignore: ' + ' '.join(bi.cbpreinstall_list) + '\n')
     rpmlist.append('runscripts: ' + ' '.join(bi.runscripts_list) + '\n')
 
     rpmlist_file = NamedTemporaryFile(prefix='rpmlist.')
@@ -787,7 +797,7 @@ def main(apiurl, opts, argv):
     rpmlist_file.flush()
 
     subst = { 'repo': repo, 'arch': arch, 'project' : prj, 'package' : pacname }
-    vm_options = ''
+    vm_options = []
     # XXX check if build-device present
     my_build_device = ''
     if config['build-device']:
@@ -805,11 +815,11 @@ def main(apiurl, opts, argv):
         else:
             my_build_swap = build_root + '/swap'
 
-        vm_options = '--vm-type=%s'%vm_type
+        vm_options = [ '--vm-type=%s'%vm_type ]
         if vm_type != 'lxc':
-            vm_options += ' --vm-disk=' + my_build_device
-            vm_options += ' --vm-swap=' + my_build_swap
-            vm_options += ' --logfile=%s/.build.log' % build_root
+            vm_options += [ '--vm-disk=' + my_build_device ]
+            vm_options += [ '--vm-swap=' + my_build_swap ]
+            vm_options += [ '--logfile=%s/.build.log' % build_root ]
             if vm_type == 'kvm':
                 if os.access(build_root, os.W_OK) and os.access('/dev/kvm', os.W_OK):
                     # so let's hope there's also an fstab entry
@@ -817,45 +827,47 @@ def main(apiurl, opts, argv):
             build_root += '/.mount'
 
         if config['build-memory']:
-            vm_options += ' --memory ' + config['build-memory']
+            vm_options += [ '--memory=' + config['build-memory'] ]
         if config['build-vmdisk-rootsize']:
-            vm_options += ' --vmdisk-rootsize ' + config['build-vmdisk-rootsize']
+            vm_options += [ '--vmdisk-rootsize=' + config['build-vmdisk-rootsize'] ]
         if config['build-vmdisk-swapsize']:
-            vm_options += ' --vmdisk-swapsize ' + config['build-vmdisk-swapsize']
+            vm_options += [ '--vmdisk-swapsize=' + config['build-vmdisk-swapsize'] ]
 
     if opts.preload:
         print "Preload done for selected repo/arch."
         sys.exit(0)
 
     print 'Running build'
-    cmd = '"%s" --root="%s" --rpmlist="%s" --dist="%s" %s --arch=%s %s %s "%s"' \
-                 % (config['build-cmd'],
-                    build_root,
-                    rpmlist_filename,
-                    bc_filename,
-                    specialcmdopts,
-                    bi.buildarch,
-                    vm_options,
-                    ' '.join(buildargs),
-                    build_descr)
+    cmd = [ config['build-cmd'], '--root='+build_root,
+                    '--rpmlist='+rpmlist_filename,
+                    '--dist='+bc_filename,
+                    '--arch='+bi.buildarch ]
+    cmd += specialcmdopts + vm_options + buildargs
+    cmd += [ build_descr ]
 
     if need_root:
-        if config['su-wrapper'].startswith('su '):
-            tmpl = '%s \'%s\''
+        sucmd = config['su-wrapper'].split()
+        if sucmd[0] == 'su':
+            if sucmd[-1] == '-c':
+                sucmd.pop()
+            cmd = sucmd + ['-s', cmd[0], 'root', '--' ] + cmd[1:]
         else:
-            tmpl = '%s %s'
-        cmd = tmpl % (config['su-wrapper'], cmd)
+            cmd = sucmd + cmd
 
     # change personality, if needed
-    if hostarch != bi.buildarch:
-        cmd = (change_personality.get(bi.buildarch, '') + ' ' + cmd).strip()
+    if hostarch != bi.buildarch and bi.buildarch in change_personality:
+        cmd = [ change_personality[bi.buildarch] ] + cmd;
 
-    print cmd
-    rc = subprocess.call(cmd, shell=True)
-    if rc:
-        print
-        print 'The buildroot was:', build_root
-        sys.exit(rc)
+    try:
+        rc = subprocess.call(cmd)
+        if rc:
+            print
+            print 'The buildroot was:', build_root
+            sys.exit(rc)
+    except KeyboardInterrupt, i:
+        print "keyboard interrupt, killing build ..."
+        subprocess.call(cmd + ["--kill"])
+        raise i
 
     pacdir = os.path.join(build_root, '.build.packages')
     if os.path.islink(pacdir):
