@@ -60,12 +60,22 @@ except:
     except:
         pass
 
+def _get_processors():
+    """
+    get number of processors (online) based on
+    SC_NPROCESSORS_ONLN (returns 1 if config name does not exist).
+    """
+    try:
+        return os.sysconf('SC_NPROCESSORS_ONLN')
+    except ValueError, e:
+        return 1
+
 DEFAULTS = { 'apiurl': 'https://api.opensuse.org',
              'user': 'your_username',
              'pass': 'your_password',
              'passx': '',
              'packagecachedir': '/var/tmp/osbuild-packagecache',
-             'su-wrapper': 'su -c',
+             'su-wrapper': 'sudo',
 
              # build type settings
              'build-cmd': '/usr/bin/build',
@@ -78,13 +88,14 @@ DEFAULTS = { 'apiurl': 'https://api.opensuse.org',
              'build-vmdisk-rootsize': '', # optional for VM builds
              'build-vmdisk-swapsize': '', # optional for VM builds
 
-             'build-jobs': os.sysconf('SC_NPROCESSORS_ONLN'), # compile with N jobs
+             'build-jobs': _get_processors(),
              'builtin_signature_check': '1', # by default use builtin check for verify pkgs
              'icecream': '0',
 
              'debug': '0',
              'http_debug': '0',
              'http_full_debug': '0',
+             'http_retries': '3',
              'verbose': '1',
              'traceback': '0',
              'post_mortem': '0',
@@ -103,12 +114,11 @@ DEFAULTS = { 'apiurl': 'https://api.opensuse.org',
              'getpac_default_project': 'openSUSE:Factory',
              # alternate filesystem layout: have multiple subdirs, where colons were.
              'checkout_no_colon': '0',
-             # local files to ignore with status, addremove, ....
+             # change filesystem layout: avoid checkout from within a proj or package dir.
+             'checkout_rooted': '0',
              # local files to ignore with status, addremove, ....
              'exclude_glob': '.osc CVS .svn .* _linkerror *~ #*# *.orig *.bak *.changes.vctmp.*',
-             # keep passwords in plaintext. If you see this comment, your osc
-             # already uses the encrypted password, and only keeps them in plain text
-             # for backwards compatibility. Default will change to 0 in future releases.
+             # whether to keep passwords in plaintext.
              'plaintext_passwd': '1',
              # limit the age of requests shown with 'osc req list'.
              # this is a default only, can be overridden by 'osc req list -D NNN'
@@ -126,9 +136,11 @@ DEFAULTS = { 'apiurl': 'https://api.opensuse.org',
              'submitrequest_accepted_template': '',
              'submitrequest_declined_template': '',
              'linkcontrol': '0',
+             'include_request_from_project': '1',
 
              # Maintenance defaults to OBS instance defaults
              'maintained_attribute': 'OBS:Maintained',
+             'maintenance_attribute': 'OBS:MaintenanceProject',
              'maintained_update_project_attribute': 'OBS:UpdateProject',
              'show_download_progress': '0',
 }
@@ -138,8 +150,8 @@ DEFAULTS = { 'apiurl': 'https://api.opensuse.org',
 config = DEFAULTS.copy()
 
 boolean_opts = ['debug', 'do_package_tracking', 'http_debug', 'post_mortem', 'traceback', 'check_filelist', 'plaintext_passwd',
-    'checkout_no_colon', 'check_for_request_on_action', 'linkcontrol', 'show_download_progress', 'request_show_interactive',
-    'use_keyring', 'gnome_keyring', 'no_verify', 'builtin_signature_check', 'http_full_debug']
+    'checkout_no_colon', 'checkout_rooted', 'check_for_request_on_action', 'linkcontrol', 'show_download_progress', 'request_show_interactive',
+    'use_keyring', 'gnome_keyring', 'no_verify', 'builtin_signature_check', 'http_full_debug', 'include_request_from_project']
 
 api_host_options = ['user', 'pass', 'passx', 'aliases', 'http_headers', 'email', 'sslcertck', 'cafile', 'capath', 'trusted_prj']
 
@@ -157,7 +169,8 @@ apiurl = %(apiurl)s
 #su-wrapper = %(su-wrapper)s
 
 # rootdir to setup the chroot environment
-# can contain %%(repo)s, %%(arch)s, %%(project)s and %%(package)s for replacement, e.g.
+# can contain %%(repo)s, %%(arch)s, %%(project)s, %%(package)s and %%(apihost)s (apihost is the hostname
+# extracted from currently used apiurl) for replacement, e.g.
 # /srv/oscbuild/%%(repo)s-%%(arch)s or
 # /srv/oscbuild/%%(repo)s-%%(arch)s-%%(project)s-%%(package)s
 #build-root = %(build-root)s
@@ -216,14 +229,16 @@ apiurl = %(apiurl)s
 # alternate filesystem layout: have multiple subdirs, where colons were.
 #checkout_no_colon = %(checkout_no_colon)s
 
+# change filesystem layout: avoid checkout within a project or package dir.
+#checkout_rooted = %(checkout_rooted)s
+
 # local files to ignore with status, addremove, ....
 #exclude_glob = %(exclude_glob)s
 
-# keep passwords in plaintext. If you see this comment, your osc
-# already uses the encrypted password, and only keeps them in plain text
-# for backwards compatibility. Default will change to 0 in future releases.
-# You can remove the plaintext password without harm, if you do not need
-# backwards compatibility.
+# keep passwords in plaintext.
+# Set to 0 to obfuscate passwords. It's no real security, just
+# prevents most people from remembering your password if they watch
+# you editing this file.
 #plaintext_passwd = %(plaintext_passwd)s
 
 # limit the age of requests shown with 'osc req list'.
@@ -237,6 +252,9 @@ apiurl = %(apiurl)s
 # show HTTP traffic useful for debugging
 #http_debug = 1
 
+# number of retries on HTTP transfer
+#http_retries = 3
+
 # Skip signature verification of packages used for build.
 #no_verify = 1
 
@@ -261,13 +279,13 @@ apiurl = %(apiurl)s
 
 # template for an accepted submitrequest
 #submitrequest_accepted_template = Hi %%(who)s,\\n
-# thanks for working on:\\t%%(dst_project)s/%%(dst_package)s.
+# thanks for working on:\\t%%(tgt_project)s/%%(tgt_package)s.
 # SR %%(reqid)s has been accepted.\\n\\nYour maintainers
 
 # template for a declined submitrequest
 #submitrequest_declined_template = Hi %%(who)s,\\n
 # sorry your SR %%(reqid)s (request type: %%(type)s) for
-# %%(dst_project)s/%%(dst_package)s has been declined because...
+# %%(tgt_project)s/%%(tgt_package)s has been declined because...
 
 #review requests interactively (default: off)
 #request_show_review = 1
@@ -278,7 +296,6 @@ apiurl = %(apiurl)s
 [%(apiurl)s]
 user = %(user)s
 pass = %(pass)s
-passx = %(passx)s
 # set aliases for this apiurl
 # aliases = foo, bar
 # email used in .changes, unless the one from osc meta prj <user> will be used
@@ -286,6 +303,8 @@ passx = %(passx)s
 # additional headers to pass to a request, e.g. for special authentication
 #http_headers = Host: foofoobar,
 #       User: mumblegack
+# Plain text password
+#pass =
 # Force using of keyring for this API
 #keyring = 1
 """
@@ -394,6 +413,17 @@ def _build_opener(url):
                 self.retried = 0
                 return response
 
+            def http_error_404(self, *args):
+                self.retried = 0
+                return None
+
+        authhandler_class = OscHTTPBasicAuthHandler
+    elif sys.version_info >= (2, 6, 6) and sys.version_info < (2, 7, 1):
+        class OscHTTPBasicAuthHandler(urllib2.HTTPBasicAuthHandler):
+            def http_error_404(self, *args):
+                self.reset_retry_count()
+                return None
+
         authhandler_class = OscHTTPBasicAuthHandler
     elif sys.version_info >= (2, 6, 5) and sys.version_info < (2, 6, 6):
         # workaround for broken urllib2 in python 2.6.5: wrong credentials
@@ -592,7 +622,6 @@ def write_initial_config(conffile, entries, custom_template = ''):
     conf_template = custom_template or new_conf_template
     config = DEFAULTS.copy()
     config.update(entries)
-    config['passx'] = base64.b64encode(config['pass'].encode('bz2'))
     # at this point use_keyring and gnome_keyring are str objects
     if config['use_keyring'] == '1' and GENERIC_KEYRING:
         protocol, host = \
@@ -613,6 +642,9 @@ def write_initial_config(conffile, entries, custom_template = ''):
         config['passx'] = ''
     if not config['plaintext_passwd']:
         config['pass'] = ''
+    else:
+        config['passx'] = base64.b64encode(config['pass'].encode('bz2'))
+
     sio = StringIO.StringIO(conf_template.strip() % config)
     cp = OscConfigParser.OscConfigParser(DEFAULTS)
     cp.readfp(sio)
@@ -666,7 +698,11 @@ def add_section(filename, url, user, passwd):
         cp.set(url, 'user', user)
         if not config['plaintext_passwd']:
             cp.remove_option(url, 'pass')
-        cp.set(url, 'passx', base64.b64encode(passwd.encode('bz2')))
+            cp.set(url, 'passx', base64.b64encode(passwd.encode('bz2')))
+        else:
+            cp.remove_option(url, 'passx')
+            cp.set(url, 'pass', passwd)
+
     file = open(filename, 'w')
     cp.write(file, True)
     if file: file.close()
@@ -765,26 +801,42 @@ def get_config(override_conffile = None,
             except gnomekeyring.NoMatchError:
                 # Fallback to file based auth.
                 pass
+
+        if not user is None and len(user) == 0:
+            user = None
+            print >>sys.stderr, 'Warning: blank user in the keyring for the ' \
+                'apiurl %s.\nPlease fix your keyring entry.'
+
         # Read credentials from config
         if user is None:
             #FIXME: this could actually be the ideal spot to take defaults
             #from the general section.
             user         = cp.get(url, 'user', raw=True) # need to set raw to prevent '%' expansion
             password     = cp.get(url, 'pass', raw=True) # especially on password!
-            passwordx    = cp.get(url, 'passx', raw=True) # especially on password!
-            if password is None or password == 'your_password':
-                try:
-                    password = passwordx.decode('base64').decode('bz2')
-                except:
-                    print "%s: no credentials known" % url
-                    password = 'your_password'
-            elif password != passwordx.decode('base64').decode('bz2'):
-                if not passwordx:
-                    # passx not present
-                    print '%s: rewriting from plain pass to encoded pass\n' % url
+            try:
+                passwordx = cp.get(url, 'passx', raw=True).decode('base64').decode('bz2') # especially on password!
+            except:
+                passwordx = ''
+            
+            if password == None or password == 'your_password':
+                password = ''
+
+            if user is None or user == '':
+                raise oscerr.ConfigError('user is blank for %s, please delete of complete the "user=" entry in %s.' % (apiurl,config['conffile']), config['conffile'])
+
+            if config['plaintext_passwd'] and passwordx or not config['plaintext_passwd'] and password:
+                if config['plaintext_passwd']:
+                    if password != passwordx:
+                        print >>sys.stderr, '%s: rewriting from encoded pass to plain pass' % url
+                    add_section(conffile, url, user, passwordx)
+                    password = passwordx
                 else:
-                    print '%s: pass and passx mismatch (rewriting from plain pass to encoded pass)\n' % url
-                add_section(conffile, url, user, password)
+                    if password != passwordx:
+                        print >>sys.stderr, '%s: rewriting from plain pass to encoded pass' % url
+                    add_section(conffile, url, user, password)
+
+            if not config['plaintext_passwd']:
+                password = passwordx
 
         if cp.has_option(url, 'http_headers'):
             http_headers = cp.get(url, 'http_headers')
