@@ -48,6 +48,12 @@ class ExtRuntimeError(OscBaseError):
         self.msg = msg
         self.file = fname
 
+class ServiceRuntimeError(OscBaseError):
+    """Exception raised when there is source service error runtime error"""
+    def __init__(self, msg):
+        OscBaseError.__init__(self)
+        self.msg = msg
+
 class WrongArgs(OscBaseError):
     """Exception raised by the cli for wrong arguments usage"""
 
@@ -101,6 +107,16 @@ class OscIOError(OscBaseError):
         self.e = e
         self.msg = msg
 
+class PackageNotInstalled(OscBaseError):
+    """
+    Exception raised when a package is not installed on local system
+    """
+    def __init__(self, pkg):
+        OscBaseError.__init__(self, pkg)
+
+    def __str__(self):
+        return 'Package %s is required for this operation' % ''.join(self.args)
+
 class SignalInterrupt(Exception):
     """Exception raised on SIGTERM and SIGHUP."""
 
